typedef enum {_taLeft, _taCenter, _taRight} TTextAlign;
typedef enum {_tavTop, _tavMiddle, _tavBottom} TTextAlignVertical;

typedef struct Screen TScreen;

typedef struct {
  unsigned int X_Min;
  unsigned int X_Max;
  unsigned int Y_Min;
  unsigned int Y_Max;
  char         Rotate;
} TTPConstants;

typedef struct  Button {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          *Caption;
  TTextAlign    TextAlign;
  TTextAlignVertical TextAlignVertical;
  const char    *FontName;
  unsigned int  Font_Color;
  char          VerticalText;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TButton;

typedef struct  Label {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          *Caption;
  const char    *FontName;
  unsigned int  Font_Color;
  char          VerticalText;
  char          Visible;
  char          Active;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TLabel;

typedef struct  Image {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  const char    *Picture_Name;
  char          Visible;
  char          Active;
  char          Picture_Type;
  char          Picture_Ratio;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TImage;

typedef struct  Circle {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Radius;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TCircle;

typedef struct  Box {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TBox;

typedef struct  Line {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  First_Point_X;
  unsigned int  First_Point_Y;
  unsigned int  Second_Point_X;
  unsigned int  Second_Point_Y;
  char          Pen_Width;
  char          Visible;
  unsigned int  Color;
} TLine;

struct Screen {
  unsigned int           Color;
  unsigned int           Width;
  unsigned int           Height;
  unsigned int           ObjectsCount;
  unsigned int           ButtonsCount;
  TButton                * const code *Buttons;
  unsigned int           LabelsCount;
  TLabel                 * const code *Labels;
  unsigned int           ImagesCount;
  TImage                 * const code *Images;
  unsigned int           CirclesCount;
  TCircle                * const code *Circles;
  unsigned int           BoxesCount;
  TBox                   * const code *Boxes;
  unsigned int           LinesCount;
  TLine                  * const code *Lines;
};

extern   TScreen                home;
extern   TBox                   Box1;
extern   TLabel                 MainLabel;
extern   TButton               bRedOn;
extern   TButton               bRedOff;
extern   TButton               bGreenOn;
extern   TButton               bGreenOff;
extern   TButton               bBlueOn;
extern   TButton               bBlueOff;
extern   TButton               bRedBlink;
extern   TButton               bGreenBlink;
extern   TButton               bBlueBlink;
extern   TLine                  TopLine;
extern   TCircle                LedRedIndicator;
extern   TCircle                LedGreenIndicator;
extern   TCircle                LedBlueIndicator;
extern   TLabel                 l_ID;
extern   TLabel                 l_DLC;
extern   TLabel                 l_DATA;
extern   TCircle                CANRXIndicator;
extern   TLabel                 l_CANRX;
extern   TBox                   dynamicbox_1;
extern   TLabel                 rcvdID;
extern   TLabel                 rcvdDLC;
extern   TLabel                 rcvdDB0;
extern   TLabel                 rcvdDB1;
extern   TLabel                 rcvdDB2;
extern   TLabel                 rcvdDB3;
extern   TLabel                 rcvdDB4;
extern   TLabel                 rcvdDB5;
extern   TLabel                 rcvdDB6;
extern   TLabel                 rcvdDB7;
extern   TCircle                CANTXIndicator;
extern   TLabel                 l_CANTX;
extern   TLabel                 l_text_redled;
extern   TLabel                 Label1;
extern   TLabel                 Label2;
extern   TLabel                 Label3;
extern   TLabel                 Label4;
extern   TLabel                 Label5;
extern   TLine                  Line1;
extern   TImage               Image1;
extern   TLabel                 Label6;
extern   TButton                * const code home_Buttons[9];
extern   TLabel                 * const code home_Labels[23];
extern   TImage                 * const code home_Images[1];
extern   TCircle                * const code home_Circles[5];
extern   TBox                   * const code home_Boxes[2];
extern   TLine                  * const code home_Lines[2];


/////////////////////////
// Events Code Declarations
void BlueBlink();
void BlueOff();
void BlueOn();
void GreenBlink();
void GreenOff();
void GreenOn();
void RedBlink();
void RedOff();
void RedOn();
/////////////////////////

/////////////////////////////////
// Caption variables Declarations
extern char Box1_Caption[];
extern char MainLabel_Caption[];
extern char bRedOn_Caption[];
extern char bRedOff_Caption[];
extern char bGreenOn_Caption[];
extern char bGreenOff_Caption[];
extern char bBlueOn_Caption[];
extern char bBlueOff_Caption[];
extern char bRedBlink_Caption[];
extern char bGreenBlink_Caption[];
extern char bBlueBlink_Caption[];
extern char TopLine_Caption[];
extern char LedRedIndicator_Caption[];
extern char LedGreenIndicator_Caption[];
extern char LedBlueIndicator_Caption[];
extern char l_ID_Caption[];
extern char l_DLC_Caption[];
extern char l_DATA_Caption[];
extern char CANRXIndicator_Caption[];
extern char l_CANRX_Caption[];
extern char dynamicbox_1_Caption[];
extern char rcvdID_Caption[];
extern char rcvdDLC_Caption[];
extern char rcvdDB0_Caption[];
extern char rcvdDB1_Caption[];
extern char rcvdDB2_Caption[];
extern char rcvdDB3_Caption[];
extern char rcvdDB4_Caption[];
extern char rcvdDB5_Caption[];
extern char rcvdDB6_Caption[];
extern char rcvdDB7_Caption[];
extern char CANTXIndicator_Caption[];
extern char l_CANTX_Caption[];
extern char l_text_redled_Caption[];
extern char Label1_Caption[];
extern char Label2_Caption[];
extern char Label3_Caption[];
extern char Label4_Caption[];
extern char Label5_Caption[];
extern char Line1_Caption[];
extern char Image1_Caption[];
extern char Label6_Caption[];
/////////////////////////////////

void DrawScreen(TScreen *aScreen);
void DrawButton(TButton *aButton);
void DrawLabel(TLabel *ALabel);
void DrawImage(TImage *AImage);
void DrawCircle(TCircle *ACircle);
void DrawBox(TBox *ABox);
void DrawLine(TLine *Aline);
void Check_TP();
void Start_TP();
void Process_TP_Press(unsigned int X, unsigned int Y);
void Process_TP_Up(unsigned int X, unsigned int Y);
void Process_TP_Down(unsigned int X, unsigned int Y);