/*
 * Project name:
     mmp_leddemo.mcpar
 * Generated by:
     Visual TFT
 * Date of creation
     1-1-2018
 * Test configuration:
     MCU:             STM32F746ZG
     Dev.Board:       Mikromedia_Plus_for_STM32F746ZG_ARM_JE
     Oscillator:      216000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */

#include "mmp_leddemo_objects.h"
#include "types.h"
#include "circularbuffer.h"
#include "CAN.h"
#include "mmp_leddemo_helpers.h"

// Lets rock...
void main() {

  // local CAN message holder
  CANmsg_t CANmsg;

  // No blinking of leds
  blinkLED_R = 0;
  blinkLED_G = 0;
  blinkLED_B = 0;
  
  // CAN display leds off
  ledstatusCANTX = 0;
  ledstatusCANRX = 0;
  
  // Default blink speed for the RGB (display) leds
  blinkspeedRED   = 5;
  blinkspeedGREEN = 5;
  blinkspeedBLUE  = 5;
  
  // Create the CAN buffer;
  ringBufS_init(&ringBufferCAN);
  ringBufS_flush(&ringBufferCAN, 1);
  
  // Send CAN message every second.
  T100ms[_T100ms_CAN] = 10;

  // Setup the CAN bus
  Init_CAN();

  // Setup the designed display
  Start_TP();
  
  // All interrupts on.
  EnableInterrupts();

  // endless loop...
  while (1) {

    // Handle display
    Check_TP();
    
    // Write a CAN message every second
    if (T100ms[_T100ms_CAN] == 0)
    {
       T100ms[_T100ms_CAN] = 10;
       WriteCAN_Message();
    }

    // Handle CANTX led in display
    // The led is triggered in teh CAN-ISR
    if ( (ledstatusCANTX == 0) && (T100ms[_T100ms_CAN_TxIndicator]) )
    {
        ledstatusCANTX = 1;
        hlpCAN_TxIndicatorOn();
    }
    else if ( (ledstatusCANTX == 1) && (T100ms[_T100ms_CAN_TxIndicator] == 0) )
    {
        ledstatusCANTX = 0;
        hlpCAN_TxIndicatorOff();
    }
    
    // Handle CANRX led in display
    if ( (ledstatusCANRX == 0) && (T100ms[_T100ms_CAN_RxIndicator]) )
    {
        ledstatusCANRX = 1;
        hlpCAN_RxIndicatorOn();
    }
    else if ( (ledstatusCANRX == 1) && (T100ms[_T100ms_CAN_RxIndicator] == 0) )
    {
        ledstatusCANRX = 0;
        hlpCAN_RxIndicatorOff();
    }

    // CAN data available in the buffer?
    if (ringBufS_pop(&ringBufferCAN, &CANmsg) != -1)
    {
       // Update CAN reception info in the display.
       hlpDisplayReceivedCAN_Data(CANmsg.cmsgID , CANmsg.cmsgDLC , CANmsg.cmsgData);
       
       // Handling the appropriate CAN message
       if (CANmsg.cmsgID == 0xAFF1)  // <-- blink speed of the RED led
       {
          blinkspeedRED = (uint32_t)CANmsg.cmsgData[0];
       }
       else if (CANmsg.cmsgID == 0xAFF2) // <-- blink speed of the GREEN led
       {
          blinkspeedGREEN = (uint32_t)CANmsg.cmsgData[0];
       }
       else if (CANmsg.cmsgID == 0xAFF3) // <-- blink speed of the BLUE led
       {
          blinkspeedBLUE = (uint32_t)CANmsg.cmsgData[0];
       }
       else if (CANmsg.cmsgID == 0xAFF0) // <-- blink speed defaults for all leds
       {
          blinkspeedRED   = (uint32_t)5;
          blinkspeedGREEN = (uint32_t)5;
          blinkspeedBLUE  = (uint32_t)5;
       }
    }

    // Handle blinking of (display) RED led
    if (blinkLED_R)
    {
       if (T100ms[_T100ms_LED_R] == 0)
       {
          LED_R ^= 1;
          T100ms[_T100ms_LED_R] = blinkspeedRED;
          
          // Update the display led (RED)
          if (LED_R)
          {
            hlpRedLedOn();
          }
          else
          {
            hlpRedLedOff();
          }
       }
    }
    
    // Handle blinking of (display) GREEN led
    if (blinkLED_G)
    {
       if (T100ms[_T100ms_LED_G] == 0)
       {
          LED_G ^= 1;
          T100ms[_T100ms_LED_G] = blinkspeedGREEN;

          // Update the display led (GREEN)
          if (LED_G)
          {
            hlpGreenLedOn();
          }
          else
          {
            hlpGreenLedOff();
          }
       }
    }
    
    // Handle blinking of (display) BLUE led
    if (blinkLED_B)
    {
       if (T100ms[_T100ms_LED_B] == 0)
       {
          LED_B ^= 1;
          T100ms[_T100ms_LED_B] = blinkspeedBLUE;
          
          // Update the display led (BLUE)
          if (LED_B)
          {
            hlpBlueLedOn();
          }
          else
          {
            hlpBlueLedOff();
          }
       }
    }
  }
}